#!/usr/bin/perl -w 

# March 2006
# To find out where perl lives on your machine, type
#	which perl
# and adjust the first line of this script accordingly.
#
# This script creates a parameter file which contains the haplotype frequencies
# for the trait locus and the descent marker.  The trait susceptibility
# allele has frequency <freq> in the population, and the descent markers has
# <nalleles> alleles in the dataset.
# Output is written to file simdata.dat.

use strict;
use Getopt::Long;

our ($opt_h, $opt_m);

sub init() {
  GetOptions("h|help"=> \$opt_h, "m|mega2"=>\$opt_m);
  usage() if $opt_h;
  usage() unless @ARGV;
}

sub usage() {
  print STDERR << "EOF";

Usage:  $0 [-h|-help] [-m|-mega2] <freq> <nalleles>
	-h|-help    : show this help message
	-m|-mega2   : format for input into mega2
Output file (default):  simdata.dat
Output files (mega2):   simdata.dat, datain1.slk, datain2.slk, map1.slk, map2.slk

EOF
	exit;
}

##############################################################################
init();

die "Invalid allele frequency.\n" if ($ARGV[0]>1 or $ARGV[0]<0);
my ($output, $mega21, $map1, $mega22, $map2);
if ($opt_m)  {
  $mega21="datain1.slk";
  $mega22="datain2.slk";
  rename $mega21, "datain1.slk_old" if -e $mega21;
  rename $mega22, "datain2.slk_old" if -e $mega22;
  $map1="map1.slk";
  $map2="map2.slk";
  rename $map1, "map1.slk_old" if -e $map1;
  rename $map2, "map2.slk_old" if -e $map2;
}
$output="simdata.dat";
rename $output, "simdata.dat_old" if -e $output;

open OUT, ">$output" or die "Cannot create file $output:  $!\n";
print OUT "2 0 0 5 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM\n";
print OUT "0 0.0 0.0 0  << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n";
print OUT "1 2\n";
print OUT "1   2  # TRAIT\n";
print OUT  1-$ARGV[0] . " " . $ARGV[0] . "\n";
print OUT "1 << NO. OF LIABILITY CLASSES  <- EDIT THIS PART \n";
print OUT "0.01 0.99 0.99  << PENETRANCE  <- EDIT THIS PART\n";
print OUT "3 $ARGV[1]  # DESCENT\n";
my $freq = 1/$ARGV[1];
print OUT "$freq " x $ARGV[1] . "\n";
print OUT "0 0  << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n";
print OUT "0.0000  << RECOMBINATION VALUES\n";
print OUT "1 0.01000 0.50000 << REC VARIED, INCREMENT, FINISHING VALUE\n";

close OUT;

if ($opt_m) {
  open MEGA, ">$mega21" or die "Cannot create file $mega21:  $!\n";

  print MEGA "2 0 0 5 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM\n";
  print MEGA "0 0.0 0.0 0  << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n";
  print MEGA "1 2 \n";
  print MEGA "1   2  # TRAIT\n";
  print MEGA 1-$ARGV[0] . " " . $ARGV[0] . "\n";
  print MEGA "1 << NO. OF LIABILITY CLASSES  <- EDIT THIS PART \n";
  print MEGA "0.01 0.99 0.99  << PENETRANCE  <- EDIT THIS PART\n";
  print MEGA "3 $ARGV[1]  # DESCENT\n";
  my $freq = 1/$ARGV[1];
  print MEGA "$freq " x $ARGV[1] . "\n";
  print MEGA "0 0  << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n";
  print MEGA "0.0000 << RECOMBINATION VALUES\n";
  print MEGA "1 0.01000 0.50000 << REC VARIED, INCREMENT, FINISHING VALUE\n";
  close MEGA;

  open MAP, ">$map1" or die "Cannot create file $map1:  $!\n";
  print MAP "CHROMOSOME HALDANE NAME\n";
  print MAP "1 0 DESCENT\n";
  close MAP;

  open MEGA2, ">$mega22" or die "Cannot create file $mega22:  $!\n";

  print MEGA2 "3 0 0 5 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM\n";
  print MEGA2 "0 0.0 0.0 0  << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n";
  print MEGA2 "1 2 3\n";
  print MEGA2 "1   2  # TRAIT\n";
  print MEGA2 1-$ARGV[0] . " " . $ARGV[0] . "\n";
  print MEGA2 "1 << NO. OF LIABILITY CLASSES  <- EDIT THIS PART \n";
  print MEGA2 "0.01 0.99 0.99  << PENETRANCE  <- EDIT THIS PART\n";
  print MEGA2 "3   2  # TRAITPROXY\n";
  print MEGA2 1-$ARGV[0] . " " . $ARGV[0] . "\n";
  print MEGA2 "3 $ARGV[1]  # DESCENT\n";
  print MEGA2 "$freq " x $ARGV[1] . "\n";
  print MEGA2 "0 0  << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n";
  print MEGA2 "0.0000 0.0000 << RECOMBINATION VALUES\n";
  print MEGA2 "1 0.01000 0.50000 << REC VARIED, INCREMENT, FINISHING VALUE\n";
  close MEGA2;

  open MAP2, ">$map2" or die "Cannot create file $map2:  $!\n";
  print MAP2 "CHROMOSOME HALDANE NAME\n";
  print MAP2 "1 0 TRAITPROXY\n";
  print MAP2 "1 0 DESCENT\n";
  close MAP2;





}
