/*Copyright 2006 Mathieu Lemire*/
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//******************************************************************************

#include "HaploDatafile.hh"
#include<iomanip>

double Haldane ( double cM ){
  return 0.5 * (1- exp(-2 * cM/100.0));
}

double Kosambi ( double cM ){  
  return 0.5 * (exp(4*cM/100.0)-1) / (exp(4*cM/100.0)+1);
}


void  HaplotypeInPosition  ( int * hhh,  int nsnp, int * nalleles, int position ){
  
  int i,j,k;
  
  int reste = position;
  int passe = 0;
  int prod=1;


  for( i=0; i<nsnp-1; i++){
    for( j=0; j<nalleles[i]; j++){
      prod=1;
      for( k = i+1; k<nsnp; k++){
	prod *= nalleles[k];
      }
      if( prod*(j+1) >= reste ){
	hhh[i] = j+1;
	reste = reste - j*prod;
	break;
      }
    }
  }

  hhh[ nsnp -1 ] = reste;

  /*  
  for(i=0;i<nsnp;i++)
    cout << hhh[i];
  cout << endl;
  */

 
}


HaploDatafile::HaploDatafile( char * filename ){

  char * ligne = new char [ L_LENGTH ];
  
  char * foo = new char [64];
  

  int alleleNumber, numberOfAlleles, locus, i,j, l, b,strpos, linenumb=0;

  double f, fpeek;

  bool read,inBlock = 0 ;

  
  ifstream FILE ( filename, ios::in );

  if( !FILE ){
    cout << filename << " : no such file \n";
    exit(-1);
  } else {
    cout << "<datafile>: Opening file " <<  filename << " for reading." << endl;
  } 


  istringstream F1;
  istringstream tampon, peektampon;

  nHBlocks = 0;

  traitLocusPosition = -1;

  // << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1), PROGRAM
  FILE.getline( ligne, L_LENGTH ); linenumb++;
  tampon.clear();
  tampon.str(ligne);
  tampon >> nLoci >> foo >> sexlinked ;
  
  if( sexlinked ) 
    cout << "Simulating sex-linked data" << endl;


  alleleFrequencies = new double * [nLoci];
  haplotypeFrequencies = new double * [nLoci]; // at most nLoci haplotypes
  nonZeroHaplotypePositions = new NonZeroHaplotypePositions [nLoci];
  cumulHaplotypeFrequencies = new double * [nLoci]; 
  nHaplotypes = new int [nLoci];
  nNonZeroHaplotypes = new int [nLoci];
  nAlleles = new int [nLoci];
  nSNPInHaplotypes = new int [nLoci];
  locusInHaplotypeBlockNumber = new int [nLoci];
  locusPositionInItsHaplotypeBlock = new int [nLoci];
  haplotypeString = new int  ** [nLoci];
  geneticPositionsInMales = new double [nLoci];
  geneticPositionsInFemales = new double [nLoci];
  interMarkerDistances = new double [nLoci-1];
  interMarkerRecombInMales = new double [nLoci-1]; 
  interMarkerRecombInFemales = new double [nLoci-1];
  femaleToMaleDistanceRatio = new double [nLoci-1];

  for(i=0;i<nLoci;i++){
    locusInHaplotypeBlockNumber[i]=-1;
    nHaplotypes[i] = 0;
    nNonZeroHaplotypes[i] = 0;
    nSNPInHaplotypes[i]=0;
    geneticPositionsInMales[i]=0.0;
    geneticPositionsInFemales[i]=0.0;
  }
  

  // << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)
  FILE.getline( ligne, L_LENGTH ); linenumb++;
  tampon.clear();
  tampon.str( ligne );
  tampon >> foo >> foo >> foo >> haplotypes;

  // LOCUS ORDER
  FILE.getline( ligne, L_LENGTH ); linenumb++;

  cout << "  Reading locus information for " << nLoci << " markers." << endl;


  for( locus = 0; locus< nLoci; locus++){
    //cout << " " << locus+1;
    //ALLELE NUMBER, NO OF ALLELES
    FILE.getline( ligne, L_LENGTH ); linenumb++;
    
    tampon.clear();
    tampon.str( ligne );
    tampon >> alleleNumber >> numberOfAlleles;
    
    nAlleles[ locus ] = numberOfAlleles;

    alleleFrequencies[ locus ] = new double [ numberOfAlleles ];
    for( i=0; i<numberOfAlleles;i++){
      alleleFrequencies[ locus ][i] =0.0;
    }

    if( alleleNumber == 1 || alleleNumber == 0 ){
      traitLocusType = alleleNumber;

      traitLocusPosition = locus;      
      
      cout << "  Trait locus is locus number " << locus+1 << " in file " << filename << "." <<  endl;

      // is the next line a frequency line ?? Taking a peek to see if the first 
      // digit found is a 0 (which indicates a frequency line) or > 0 (which
      // indicates a penetrance line

      strpos = FILE.tellg();
      FILE.getline( ligne, L_LENGTH);
      peektampon.clear();
      peektampon.str(ligne);
      peektampon >> fpeek;
      FILE.seekg( strpos );

      // Here a trait locus was found.  If the TL is not part of a haplo block,
      // then the next line should be a frequency line
      // Otherwise, if the TL is in a block, the next line will either be another "alleleNumber numberOfAlleles"
      // or else a nLiabilityClasses

      if( fpeek >= 0.0 &&  fpeek < 1.0 ){ // then the next line is a frequency line 

	if( !inBlock ){ // then this is a single locus
	  
	  nHBlocks++;
	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  nHaplotypes[ nHBlocks - 1] = nAlleles[ locus ];
	  
	  nSNPInHaplotypes[ nHBlocks -1 ]++;
	  locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  haplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  cumulHaplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  haplotypeString[  nHBlocks -1  ] = new int * [  nHaplotypes[ nHBlocks -1]  ];
	  
	  int * nal = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
	  for(i=0;i< nSNPInHaplotypes[ nHBlocks -1 ];i++){
	    nal[i] = nAlleles[ locus -  nSNPInHaplotypes[ nHBlocks -1 ] + i + 1 ];
	  }
	 	  
	  i=0;
	  read = 1;	
	  
	  nonZeroHaplotypePositions[  nHBlocks -1 ].Set(  nHaplotypes[ nHBlocks - 1] );

	  while( i< nHaplotypes[ nHBlocks - 1] ){

	    haplotypeString[  nHBlocks -1  ][i] = new int [  nSNPInHaplotypes[ nHBlocks -1 ] ];
	    
	    if( read ){
	      FILE.getline( ligne, L_LENGTH );linenumb++;
	      F1.clear();
	      F1.str( ligne );
	      read = 0;
	    }
	    if( F1 >> f ){
	      alleleFrequencies[ locus ][i] = f;
	      if( f > 0.0 ){
		nNonZeroHaplotypes[ nHBlocks -1 ]++;
		nonZeroHaplotypePositions[  nHBlocks -1 ].Add( i ); 	    
		HaplotypeInPosition( haplotypeString[  nHBlocks -1  ][i], nSNPInHaplotypes[ nHBlocks -1 ] , nal, i+1  );
	      }
	      haplotypeFrequencies[ nHBlocks -1][i] = f;
	      if( i>0 ){
		cumulHaplotypeFrequencies[ nHBlocks -1][i] = cumulHaplotypeFrequencies[ nHBlocks -1][i-1]+f;
	      } else {
		cumulHaplotypeFrequencies[ nHBlocks -1][0] = f;
	      }	   
	      i++;
	    } else {

	      // frequencies are spread over multiple lines

	      strpos = FILE.tellg();
	      FILE.getline( ligne, L_LENGTH);
	      peektampon.clear();
	      peektampon.str(ligne);
	      peektampon >> fpeek;
	      FILE.seekg( strpos );
	      
	      if( fpeek >= 0.0 &&  fpeek < 1.0 ){	
		// next line is a frequency line, keep reading
		read = 1;
	      } else {
		cout << "Problem with a frequency line. Expecting a frequency but read:  " << endl;		
		cout << ligne << endl;
		cout << "in line " << linenumb + 1 <<endl;
		exit(-1);
	      }	      
	    }
	  }
	} else { // then this is the end of a haplotype block. I am inside an if(next line is a frequency line)

	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  nSNPInHaplotypes[ nHBlocks -1 ]++;
	  locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  nHaplotypes[ nHBlocks -1 ] *= nAlleles[ locus ];
	  
	  haplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  cumulHaplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  
	  haplotypeString[  nHBlocks -1  ] = new int * [  nHaplotypes[ nHBlocks -1]  ];
	  	
	  int * nal = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
	  for(i=0;i< nSNPInHaplotypes[ nHBlocks -1 ];i++){
	    nal[i] = nAlleles[ locus -  nSNPInHaplotypes[ nHBlocks -1 ] + i + 1 ];
	  }

	  nonZeroHaplotypePositions[  nHBlocks -1 ].Set(  nHaplotypes[ nHBlocks - 1] );


	  i=0;
	  read = 1;
	  while( i< nHaplotypes[ nHBlocks - 1] ){	
	    haplotypeString[  nHBlocks -1  ][i] = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
    
	    if( read ){
	      FILE.getline( ligne, L_LENGTH );linenumb++;
	      F1.clear();
	      F1.str( ligne );
	      read = 0;
	    }
	    if( F1 >> f ){
	
	      if( f > 0.0 ){
		nNonZeroHaplotypes[ nHBlocks -1 ]++;
		nonZeroHaplotypePositions[  nHBlocks -1 ].Add( i );    
	      
	      HaplotypeInPosition( haplotypeString[  nHBlocks -1  ][i],nSNPInHaplotypes[ nHBlocks -1 ] , nal, i+1 );
	      }
	      haplotypeFrequencies[ nHBlocks -1][i] = f;
	      if( i>0 ){
		cumulHaplotypeFrequencies[ nHBlocks -1][i] = cumulHaplotypeFrequencies[ nHBlocks -1][i-1]+f;
	      } else {
		cumulHaplotypeFrequencies[ nHBlocks -1][0] = f;
	      }	   
	      i++;
	    } else {
	      // frequencies are spread over multiple lines

	      strpos = FILE.tellg();
	      FILE.getline( ligne, L_LENGTH);
	      peektampon.clear();
	      peektampon.str(ligne);
	      peektampon >> fpeek;
	      FILE.seekg( strpos );
	      
	      if( fpeek >= 0.0 &&  fpeek < 1.0 ){
		read = 1;
	      } else {
		cout << "Problem with a frequency line. Expecting a frequency but read:  " << endl;		
		cout << ligne << endl;
		cout << "in line " << linenumb + 1 <<endl;
		exit(-1);
	      }
	      
	    }
	  }

	}
	 
	// just read a frequency line, so it means I am now out of a block;
	inBlock = 0;
 
	// now the next lines are for the penetrances:

	FILE.getline( ligne, L_LENGTH );linenumb++;
	tampon.clear();
	tampon.str( ligne );

	tampon >> nLiabilityClasses;

	if( traitLocusType == 1 ){
	

	  penetrances = new double * [nLiabilityClasses];
	  penetrancesForMales = new double * [nLiabilityClasses];


	  for( i=0; i<nLiabilityClasses; i++){
	    penetrances[i] = new double [3];       
	    penetrancesForMales[i] = new double [2];
	    FILE.getline( ligne, L_LENGTH );linenumb++;
	    tampon.clear();
	    tampon.str( ligne );
	    if( !(tampon>>penetrances[i][0]>>penetrances[i][1]>>penetrances[i][2]) ){
	      cout << "Problem reading the 3 penetrance values on line " << linenumb << endl;
	      exit(-1);
	    }
	    cout << "  Read penetrances for class " << i+1 << ": ";
	   
	    cout << penetrances[i][0]<< " " << penetrances[i][1]<< " " << penetrances[i][2] << endl;
	    if( sexlinked ){
	      FILE.getline( ligne, L_LENGTH );linenumb++;
	      tampon.clear();
	      tampon.str( ligne );

	      if( !(tampon>>penetrancesForMales[i][0]>>penetrancesForMales[i][1] ) ){
		 cout << "Problem reading the 2 male penetrance values on line " << linenumb << endl;
		 exit(-1);
	      } 	      
	      cout << "  " << penetrancesForMales[i][0]<< " " << penetrancesForMales[i][1] << endl;
	    }

	  }
	} else {  // then the trait locus is a quantitative trait locus
	  
	  if( nLiabilityClasses != 1 ){
	    cout << "There should only be 1 trait in file " << filename << " in line " << linenumb << endl;
	    exit(-1);
	  } 
	  
	  genotypeMeans = new double [3];
	  FILE.getline( ligne, L_LENGTH );linenumb++;
	  tampon.clear();
	  tampon.str( ligne );
	  if( !( tampon >> genotypeMeans[0] >> genotypeMeans[1] >> genotypeMeans[2] )){
	    cout <<  "Problem reading the 3 genotype means on line " << linenumb << endl;
	    exit(-1);
	  }

	  cout << "Read genotype means: " <<  genotypeMeans[0] << " " <<  genotypeMeans[1] << " " <<  genotypeMeans[2] << endl;
	  FILE.getline( ligne, L_LENGTH );linenumb++;
	  tampon.clear();
	  tampon.str( ligne );
	  tampon >> variance;
	  cout << "Read trait variance: " << variance << endl;
	  FILE.getline( ligne, L_LENGTH );linenumb++;
	  tampon.clear();
	  tampon.str( ligne );
	  tampon >> varianceMultiplier;
	  cout << "Read variance multiplier in heterozygous: " << varianceMultiplier << endl;
	  
	}

	

      } else if( fpeek >= 1 ){ // I am reading the penetrances and I am still in or starting a block
     

	if( !inBlock ){ 

	  nHBlocks++;
	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  
	  nHaplotypes[ nHBlocks - 1] = nAlleles[ locus ];
	  nSNPInHaplotypes[ nHBlocks -1 ]++;
	  locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  inBlock = 1;
	 
	} else {

	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  nSNPInHaplotypes[ nHBlocks -1 ]++;
	  nHaplotypes[ nHBlocks -1 ] *= nAlleles[ locus ];
	  locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];

	}

	FILE.getline( ligne, L_LENGTH );linenumb++;
	tampon.clear();
	tampon.str( ligne );
	tampon >> nLiabilityClasses;


	if( traitLocusType == 1 ){

	  penetrances = new double * [nLiabilityClasses];
	  penetrancesForMales = new double * [nLiabilityClasses];
	  for( i=0; i<nLiabilityClasses; i++){
	    penetrances[i] = new double [3];
       	    penetrancesForMales[i] = new double [2];
	    FILE.getline( ligne, L_LENGTH );linenumb++;
	    tampon.clear();
	    tampon.str( ligne );
	    if( !( tampon>>penetrances[i][0]>>penetrances[i][1]>>penetrances[i][2] )){
	      cout << "Problem reading the 3 penetrance values on line " << linenumb << endl;
	      exit(-1);
	    }
	    cout << "  Read penetrances for class " << i+1 << ": ";
	    cout <<  penetrances[i][0]<< " " << penetrances[i][1]<< " " << penetrances[i][2] << endl;

	    if( sexlinked ){
	      FILE.getline( ligne, L_LENGTH );linenumb++;
	      tampon.clear();
	      tampon.str( ligne );
	      if( !(tampon>>penetrancesForMales[i][0]>>penetrancesForMales[i][1] ) ){
		cout << "Problem reading the 2 male penetrance values on line " << linenumb << endl;
		exit(-1);
	      } 	      
	      cout << "  " <<  penetrancesForMales[i][0]<< " " << penetrancesForMales[i][1] << endl;
	    }


	  }
	} else {
	  
	  if( nLiabilityClasses != 1 ){
	    cout << "There should only be 1 trait in file " << filename << " in line " << linenumb << endl;
	    exit(-1);
	  } 
	  
	  genotypeMeans = new double [3];
	  FILE.getline( ligne, L_LENGTH );linenumb++;
	  tampon.clear();
	  tampon.str( ligne );
	  if( !( tampon >> genotypeMeans[0] >> genotypeMeans[1] >> genotypeMeans[2] ) ){
	    cout <<  "Problem reading the 3 genotype means on line " << linenumb << endl;
	    exit(-1);
	  }
	  cout << "  Read genotype means: " <<  genotypeMeans[0] << " " <<  genotypeMeans[1] << " " <<  genotypeMeans[2] << endl;
	  FILE.getline( ligne, L_LENGTH );linenumb++;
	  tampon.clear();
	  tampon.str( ligne );
	  tampon >> variance;
	  cout << "  Read trait variance: " << variance << endl;
	  FILE.getline( ligne, L_LENGTH );linenumb++;
	  tampon.clear();
	  tampon.str( ligne );
	  tampon >> varianceMultiplier;
	  cout << "  Read variance multiplier in heterozygous: " << varianceMultiplier << endl;
	  
	}


	// NEW 110117
	// After reading the penetrances, I need to check to see if the next line is a frequency line.

	strpos = FILE.tellg();
	FILE.getline( ligne, L_LENGTH);
	peektampon.clear();
	peektampon.str(ligne);
	peektampon >> fpeek;
	FILE.seekg( strpos );
	
	if( fpeek >= 0.0 &&  fpeek < 1.0 ){ // then the next line is a frequency line.  I should already be in a block, so the block ends with the TL
	  
	  // these things that I normally set when I read a frequency line, I already set them when I read the trait locus info
	  //
	  //locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  //nSNPInHaplotypes[ nHBlocks -1 ]++;
	  //locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  //nHaplotypes[ nHBlocks -1 ] *= nAlleles[ locus ];
	  
	  haplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  cumulHaplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  
	  haplotypeString[  nHBlocks -1  ] = new int * [  nHaplotypes[ nHBlocks -1]  ];
	  	
	  int * nal = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
	  for(i=0;i< nSNPInHaplotypes[ nHBlocks -1 ];i++){
	    nal[i] = nAlleles[ locus -  nSNPInHaplotypes[ nHBlocks -1 ] + i + 1 ];
	  }
	  
	  nonZeroHaplotypePositions[  nHBlocks -1 ].Set(  nHaplotypes[ nHBlocks - 1] );


	  i=0;
	  read = 1;
	  while( i< nHaplotypes[ nHBlocks - 1] ){
	    haplotypeString[  nHBlocks -1  ][i] = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
    
	    if( read ){
	      FILE.getline( ligne, L_LENGTH );linenumb++;

	      F1.clear();
	      F1.str( ligne );
	      read = 0;
	    }
	    if( F1 >> f ){
	
	      if( f > 0.0 ){
		nNonZeroHaplotypes[ nHBlocks -1 ]++;
		nonZeroHaplotypePositions[  nHBlocks -1 ].Add( i );    
	      
	      HaplotypeInPosition( haplotypeString[  nHBlocks -1  ][i],nSNPInHaplotypes[ nHBlocks -1 ] , nal, i+1 );
	      }
	      haplotypeFrequencies[ nHBlocks -1][i] = f;
	      if( i>0 ){
		cumulHaplotypeFrequencies[ nHBlocks -1][i] = cumulHaplotypeFrequencies[ nHBlocks -1][i-1]+f;
	      } else {
		cumulHaplotypeFrequencies[ nHBlocks -1][0] = f;
	      }	   
	      i++;
	    } else {
	      // frequencies are spread over multiple lines

	      strpos = FILE.tellg();
	      FILE.getline( ligne, L_LENGTH);
	      peektampon.clear();
	      peektampon.str(ligne);
	      peektampon >> fpeek;
	      FILE.seekg( strpos );
	      
	      if( fpeek >= 0.0 &&  fpeek < 1.0 ){
		read = 1;
	      } else {
		cout << "Problem with a frequency line. Expecting a frequency but read:  " << endl;		
		cout << ligne << endl;
		cout << "in line " << linenumb + 1 <<endl;
		exit(-1);
	      }
	      
	    }
	  }





	}




      } else {
	cout << "Unexpected character found for Allele Number in line " << linenumb << endl;
	exit(-1);
      }
    } else if( alleleNumber == 3 ) {  // the allele number is not a 1, I am reading marker locus information
      
      // checking to see if I am in a block or not

      strpos = FILE.tellg();
      FILE.getline( ligne, L_LENGTH);
      peektampon.clear();
      peektampon.str(ligne);
      peektampon >> fpeek;
      FILE.seekg( strpos );
      
      if( fpeek >= 0.0 && fpeek < 1.0 ){ // the next line to be read is a frequency line


 
	if( inBlock ){
 
	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  nSNPInHaplotypes[ nHBlocks -1 ]++;
	  locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  nHaplotypes[ nHBlocks -1 ] *= nAlleles[ locus ];

	  haplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  cumulHaplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  
	  haplotypeString[  nHBlocks -1  ] = new int * [  nHaplotypes[ nHBlocks -1]  ];
	  
	  int * nal = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
	  for(i=0;i< nSNPInHaplotypes[ nHBlocks -1 ];i++){
	    nal[i] = nAlleles[ locus -  nSNPInHaplotypes[ nHBlocks -1 ] + i + 1 ];
	  }

	  nonZeroHaplotypePositions[  nHBlocks -1 ].Set(  nHaplotypes[ nHBlocks - 1] );

	  i=0;
	  read = 1;
	  while( i< nHaplotypes[ nHBlocks - 1] ){	

	    haplotypeString[  nHBlocks -1  ][i] = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
    
	    if( read ){
	      FILE.getline( ligne, L_LENGTH );linenumb++;
	      F1.clear();
	      F1.str( ligne );
	      read = 0;
	    }
	    if( F1 >> f ){
	      if( f > 0.0 ){
		nNonZeroHaplotypes[ nHBlocks -1 ]++;
		nonZeroHaplotypePositions[  nHBlocks -1 ].Add( i );
	      	     
	      
		//	       cout << i << " ";
	      HaplotypeInPosition(haplotypeString[  nHBlocks -1  ][i], nSNPInHaplotypes[ nHBlocks -1 ] , nal, i+1 );
	      }
	      haplotypeFrequencies[ nHBlocks -1][i] = f;
	      if( i>0 ){
		cumulHaplotypeFrequencies[ nHBlocks -1][i] = cumulHaplotypeFrequencies[ nHBlocks -1][i-1]+f;
	      } else {
		cumulHaplotypeFrequencies[ nHBlocks -1][0] = f;
	      }	   
	      i++;
	    } else { 
	      
	      // frequencies are spread over multiple lines

	        
	      strpos = FILE.tellg();
	      FILE.getline( ligne, L_LENGTH);
	      peektampon.clear();
	      peektampon.str(ligne);
	      peektampon >> fpeek;
	      FILE.seekg( strpos );
	      
	      if( fpeek >= 0.0 && fpeek < 1.0 ){
		read = 1;
	      } else {	
		cout << "Problem with a frequency line. Expecting a frequency but read:  " << endl;		
		cout << ligne << endl;
		cout << "in line " << linenumb + 1 <<endl;
		exit(-1);
	      }
	      
	    }
	  }


	  
	  if( F1 >> f ){ 	    
	    cout << "There may be too many frequencies in line " << linenumb << ".";
	    cout << "  Proceeding anyway..." << endl;
	  }



	} else { // not in block, so it's a marker 
	      
	  // for consistency, considering single markers as haplotypes; I should rewrite everything here
	 

	  nHBlocks++;
	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  nHaplotypes[ nHBlocks - 1] = nAlleles[ locus ];
	  nSNPInHaplotypes[ nHBlocks -1 ]++;locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  haplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  cumulHaplotypeFrequencies[ nHBlocks -1 ] = new double [ nHaplotypes[ nHBlocks -1] ];
	  haplotypeString[  nHBlocks -1  ] = new int * [  nHaplotypes[ nHBlocks -1]  ];
	  int * nal = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
	  for(i=0;i< nSNPInHaplotypes[ nHBlocks -1 ];i++){
	    nal[i] = nAlleles[ locus -  nSNPInHaplotypes[ nHBlocks -1 ] + i + 1 ];
	  }
	  
	  i=0;
	  nonZeroHaplotypePositions[  nHBlocks -1 ].Set(  nHaplotypes[ nHBlocks - 1] );

	  read = 1;
	  while( i< nHaplotypes[ nHBlocks - 1] ){	
	    haplotypeString[  nHBlocks -1  ][i] = new int [ nSNPInHaplotypes[ nHBlocks -1 ] ];
  
    
	    if( read ){
	      FILE.getline( ligne, L_LENGTH );linenumb++;
	      F1.clear();	      
	      F1.str( ligne );
	      read = 0;
	    }
	    if( F1 >> f ){
	      if( f > 0.0 ){
		nNonZeroHaplotypes[ nHBlocks -1 ]++;
		nonZeroHaplotypePositions[  nHBlocks -1 ].Add( i );  

		HaplotypeInPosition(haplotypeString[  nHBlocks -1  ][i],nSNPInHaplotypes[ nHBlocks -1 ] , nal, i+1 );
	      }	      
	     
	      alleleFrequencies[ locus ][i] = f;
	   
	      haplotypeFrequencies[ nHBlocks -1][i] = f;
	      if( i>0 ){
		cumulHaplotypeFrequencies[ nHBlocks -1][i] = cumulHaplotypeFrequencies[ nHBlocks -1][i-1]+f;
	      } else {
		cumulHaplotypeFrequencies[ nHBlocks -1][0] = f;
	      }	   
	      i++;
	    } else {
	      // frequencies are spread over multiple lines

	      strpos = FILE.tellg();
	      FILE.getline( ligne, L_LENGTH);
	      peektampon.clear();
	      peektampon.str(ligne);
	      peektampon >> fpeek;
	      FILE.seekg( strpos );
	      
	      if( fpeek >= 0.0 && fpeek < 1.0 ){
		read =1;
	      } else {
		cout << "Problem with a frequency line. Expecting a frequency but read:  " << endl;		
		cout << ligne << endl;
		cout << "in line " << linenumb + 1 <<endl;
		exit(-1);
	      }
	      
	    }
	  }
	  if( F1 >> f ){ 	    
	    cout << "There may be too many frequencies in line " << linenumb << ".";
	    cout << "Proceeding anyway..." << endl;
	  }
	}

	inBlock = 0; // end of block

      } else { // the next line are NOT  frequencies
     
	if( !inBlock ){ // I haven't read the next line yet, so I am still at locus (3 nalleles)
	  nHBlocks++;
	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  nHaplotypes[ nHBlocks - 1] = nAlleles[ locus ];
	  nSNPInHaplotypes[ nHBlocks -1 ]++;locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  inBlock = 1;
	 
	} else {
	  locusInHaplotypeBlockNumber[ locus ] = nHBlocks - 1;
	  nSNPInHaplotypes[ nHBlocks -1 ]++;locusPositionInItsHaplotypeBlock[ locus ] = nSNPInHaplotypes[ nHBlocks -1 ];
	  nHaplotypes[ nHBlocks -1 ] *= nAlleles[ locus ];
	  
	}
      }
    } else { 
      cout << "Problem with line " << linenumb + 1 << ": expecting a marker type but read"<<endl;
      cout << ligne << endl; exit(-1);
    }
  }
 
  



  
  // calculating the marker allele frequencies
  locus = 0;
  for(b=0;b<nHBlocks;b++){
    if( nSNPInHaplotypes[ b ] > 1 ){

      for( j = 0 ; j< nNonZeroHaplotypes[ b ]; j++){
	

	for( l =0 ; l< nSNPInHaplotypes[ b ]; l++){
	  i = haplotypeString[b][nonZeroHaplotypePositions[b].Positions(j) ][l];
	  alleleFrequencies[ locus+l ][ i-1] +=  haplotypeFrequencies[ b ][  nonZeroHaplotypePositions[b].Positions(j) ];
	}
      }
    }
    locus +=   nSNPInHaplotypes[ b ];

    if( cumulHaplotypeFrequencies[b][ nHaplotypes[b]-1 ] <= 0.999999 ){
      cout << "  Haplotype or allele frequencies add up to ";
      cout << setprecision( 6 ) <<   cumulHaplotypeFrequencies[b][ nHaplotypes[b]-1 ];
      cout << " in marker block " << b+1 << ". \n  These frequencies have been recalibrated to add up to 1." << endl;
    }

  }


   
  



  // << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)
  FILE.getline( ligne, L_LENGTH );linenumb++;
  tampon.clear();
  tampon.str(ligne);
  tampon >> sexDiff;

  // intermarker distances
  FILE.getline( ligne, L_LENGTH );linenumb++;
  tampon.clear();
  tampon.str(ligne);

  
  i=0;
  geneticPositionsInMales[i]=0.0;
  geneticPositionsInFemales[i]=0.0;
  chromosomeLengthInMales = 0.0;
  chromosomeLengthInFemales = 0.0;
  


  while(  i<nLoci-1 && tampon >> f ){
    // cout << i+1 << " -- " << f << "(" << Haldane(f) << "," << Kosambi(f) << ")  -- " << i+2 << endl;
    interMarkerDistances[i] = f;
    interMarkerRecombInMales[i] = Haldane( f );
    chromosomeLengthInMales += f;
    geneticPositionsInMales[i+1]=geneticPositionsInMales[i]+f;
    i++;
  }
  //cout << endl;
  if( tampon >> f ){
    cout << "Too many intermarker distances."<<endl;
    exit(-1);
  }
  if( i<nLoci-1 ){
    cout << "Not enough intermarker distances."<<endl;
    exit(-1);
  }

  if( sexDiff ){
    FILE.getline( ligne, L_LENGTH );linenumb++;
    tampon.clear();
    tampon.str(ligne);
    i=0;
    while(  i<nLoci-1 && tampon >> f ){
      femaleToMaleDistanceRatio[i] = f;
      interMarkerRecombInFemales[i] = Haldane( f*interMarkerDistances[i] );
      chromosomeLengthInFemales += f * interMarkerDistances[i];

      geneticPositionsInFemales[i+1]=geneticPositionsInFemales[i] + f * interMarkerDistances[i];

      i++;
    }
    if( tampon >> f ){
      cout << "Too many FEMALE/MALE ratios."<<endl;
      exit(-1);
    }
    if( i<nLoci-1 ){
      cout << "Not enough FEMALE/MALE ratios"<<endl;
      exit(-1);
    }
  } else {
    for( i =0; i<nLoci-1; i++){
      femaleToMaleDistanceRatio[i] = 1.0;
      interMarkerRecombInFemales[i] = interMarkerRecombInMales[i];
      chromosomeLengthInFemales += interMarkerDistances[i];
      geneticPositionsInFemales[i+1]=geneticPositionsInMales[i+1];
      
    }
  }

  FILE.close();

}
    
