/*Copyright 2006 Mathieu Lemire*/
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//******************************************************************************

#include<list>
#include<iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <math.h>
#include<stdlib.h>

#define L_LENGTH 1000000

#define DEBUG_H 0

using namespace std;

double Haldane ( double );
double Kosambi ( double );
double Poisson (int, double);

class NonZeroHaplotypePositions 
{
  int nNonZero;
  int * positions;
public:
  NonZeroHaplotypePositions(){
    nNonZero = 0;
  }

  void Set ( int ntotalhaplo ){
    positions = new int [ntotalhaplo];
  }

  void Add ( int pos ){
    positions[ nNonZero++ ] = pos;
  }

  int NNonZero (){
    return nNonZero;
  }
  int Positions( int i ){
    return positions[i];
  }

};


class HaploDatafile 
{

  int nLoci;

  // this indicates if haplotype frequencies are to be found in datafile.dat
  bool haplotypes;

  int * locusOrder;
  
  // this is where the descent marker is located
  // if negative, there is no trait locus
  int traitLocusType;
  int traitLocusPosition;
  int nLiabilityClasses;
  double ** penetrances;
  double ** penetrancesForMales;
  double * genotypeMeans;
  double variance;
  double varianceMultiplier;
  int * nAlleles;
  double ** alleleFrequencies;
  int nHBlocks;
  int * nHaplotypes; // in the nHBlocks'th haplotype block
  int *** haplotypeString; // the string of the haplotype
  int * nSNPInHaplotypes;
  double ** haplotypeFrequencies;
  double ** cumulHaplotypeFrequencies;
  double * conditionalCumul;
  bool filledConditionalCumul;
  int * nNonZeroHaplotypes;
  bool sexlinked;

  NonZeroHaplotypePositions * nonZeroHaplotypePositions;

  int * locusInHaplotypeBlockNumber;
  int * locusPositionInItsHaplotypeBlock;
  bool sexDiff;
  double * interMarkerDistances;
  double * interMarkerRecombInMales;
  double * interMarkerRecombInFemales;
  double chromosomeLengthInMales;
  double chromosomeLengthInFemales;
  double * geneticPositionsInMales;
  double * geneticPositionsInFemales;
  double * femaleToMaleDistanceRatio;

public:

  HaploDatafile( char * );

  int NLoci(){
    return nLoci;
  }
  int NHBlocks(){
    return nHBlocks;
  }
  int NHaplotypes( int block ){
    return nHaplotypes[block];
  }
  int * Haplotype( int block, int number, bool fully=0 ){
    if( number > NHaplotypes( block ) ){
      cout << "Not enough haplotypes defined in <datafile> for block " << block+1 << endl;
      if( fully ){
	cout << " in order to simulate fully informative markers." << endl;
      }
      cout << "Exiting." << endl;
      exit(1);
    } 
    return haplotypeString[ block ][number];
  }
  double HaplotypeFrequency( int block, int number ){
    return haplotypeFrequencies[block][number]; 
  }
  int NAlleles( int locus ){
    return nAlleles[ locus ];
  }
  double AlleleFrequency( int locus, int allele ){
    return alleleFrequencies[ locus ][ allele ];
  }

  int InWhichBlock ( int locus ){
    return locusInHaplotypeBlockNumber[ locus ];
  }
  int NumberOfSNPInBlock( int b ){
    return nSNPInHaplotypes[b];
  }
  int TraitLocusPosition(){
    return traitLocusPosition;
  }
  int LocusPositionInItsHaplotypeBlock(int i){
    return locusPositionInItsHaplotypeBlock[i];
  }

  bool SexLinked(){
    return sexlinked;
  }

  bool Recombine (int start, int end, int sex, double random = drand48() ){

    //    if( interMarkerDistances[ start ] <=0 ) return 0;

  
    int i;
    bool rec = 0;
    double dist;
    
    if( sex == 1 ){
      dist =  interMarkerRecombInMales[ start ];
    } else {
      dist =  interMarkerRecombInFemales[ start ];
    }

    if( random < dist ){
      rec = 1;
    }

    
    /*

    if( end-start != 1 ){
      cout << "Consecutive markers needed in HaploDatafile::Recombined()"<<endl;
      exit(-1);
    }
    if( mapfunction == 0 ){
      if( random < dist ){
	rec = 1;
      }
    } else if ( mapfunction == 1 ){
      if( random < Haldane(dist )){
	rec = 1;
      }
    } else  if ( mapfunction == 2 ){
      if( random < Kosambi(dist )){
	rec = 1;
      }
    } else {
      cout << "Unknown map function in HaploDatafile::Recombined()"<<endl;
      exit(-1);
    }
    
    */

    return rec;
  }




  /*
  int NRecombEvents ( int sex,  double random = drand48() ){
    
    int i=0;

    double length;

    if( sex == 1 ){
      length = chromosomeLengthInMales;
    } else {
      length = chromosomeLengthInFemales;
    }
    
    double cumul = Poisson( 0, length );

    while( random > cumul ){
      cumul += Poisson( ++i, length);
    }

    return i;
  }

  double * RecombinationLocations ( int sex , int n ){
    
    int i;
    double length, random;

    vector<double> temp (n);

    double * loc = new double [n];
    

    if( sex == 1 ){
      length = chromosomeLengthInMales;
    } else {
      length = chromosomeLengthInFemales;
    }
    
    for( i=0;i<n;i++){
      random = drand48();
      temp[i] = length * random;
    }
    temp.sort();
    for( i=0;i<n;i++){
      loc[i] = temp[i];
    }
    
    return loc; 


  }
  
  
  */

  int * GenerateHaplotype( int block , double random = drand48()  ){
    int i=0;
    //cout << random << endl;
    // if the frequencies don't add up to 1
    double adjrandom = random*cumulHaplotypeFrequencies[ block ][  NHaplotypes( block ) - 1  ]; 
    if( DEBUG_H ){
	cout << "Comparing " << adjrandom << " to ";
	cout << cumulHaplotypeFrequencies[ block ][nonZeroHaplotypePositions[block].Positions(i)];
	cout << " for haplotype " << nonZeroHaplotypePositions[block].Positions(i);
	cout << " in block " << block << endl;
      }
    while( adjrandom > cumulHaplotypeFrequencies[ block ][nonZeroHaplotypePositions[block].Positions(i)]){     
      i++;
      if( DEBUG_H ){
	cout << "Comparing " << adjrandom << " to ";
	cout << cumulHaplotypeFrequencies[ block ][nonZeroHaplotypePositions[block].Positions(i)];
	cout << " for haplotype " << nonZeroHaplotypePositions[block].Positions(i);
	cout << " in block " << block << endl;
      }

    }
    return  Haplotype( block, nonZeroHaplotypePositions[block].Positions(i) );
  }
 
  int * GenerateConditionalHaplotype( int block , int position, int al, double random = drand48()  ){

    //cout << block << " " << position << " " << al << endl;

    int i,j, nhap=nonZeroHaplotypePositions[block].NNonZero();
    int cur;
    
    bool found = 0;

    double * conditionalCumul = new double [ nhap ];

    i=0;
 
    cur = haplotypeString[ block ][ nonZeroHaplotypePositions[ block ].Positions( i )  ][  position -1 ];

    if( DEBUG_H ){
      cout << "Haplotype " <<  nonZeroHaplotypePositions[ block ].Positions( i );
      cout << " in block " << block;
      cout << " has freq " <<  HaplotypeFrequency( block,    nonZeroHaplotypePositions[block].Positions( i )   ) ;
      cout << endl;
    }

    if( cur == al ){
      conditionalCumul[i] = HaplotypeFrequency( block,    nonZeroHaplotypePositions[block].Positions( i )   );
    } else {
      conditionalCumul[i] = 0.0;
    }

    for( i=1; i< nhap; i++){
      if( DEBUG_H ){
	cout << "Haplotype " <<  nonZeroHaplotypePositions[ block ].Positions( i );
	cout << " in block " << block;
	cout << " has freq " <<  HaplotypeFrequency( block,    nonZeroHaplotypePositions[block].Positions( i )   ) ;
	cout << endl;
      }

     
      cur = haplotypeString[ block ][ nonZeroHaplotypePositions[ block ].Positions( i )  ][  position -1 ];

      if( cur == al  ){
	conditionalCumul[i] = conditionalCumul[i-1] + 
	  HaplotypeFrequency( block,  nonZeroHaplotypePositions[block].Positions( i ) );
      } else {
	conditionalCumul[i] = conditionalCumul[i-1];
      }
    }


    if( conditionalCumul[nhap-1] > 0 ){
      
      for(i=0;i<nhap;i++){      
	conditionalCumul[i] /= conditionalCumul[nhap-1];
      }
    } else {
      cout << "Could not find a haplotype in block " << block+1 << " with an allele number ";
      cout << al << " at position " << position  << endl;
      exit(1);
    } 



    

    i=0;
    while( random >  conditionalCumul[i] ) i++;

    delete conditionalCumul;
    return  Haplotype( block, nonZeroHaplotypePositions[block].Positions( i )  );

    

  }
  

  void PrintMap(){
    int i;
    cout << "Marker GeneticPosition" <<endl;
    for( i=0; i< nLoci; i++){
      cout << i << " " << geneticPositionsInMales[i] << " " << geneticPositionsInFemales[i] << endl;
    }

  }    

  double MarkerPosition(int i, int sex = 1 ){
    if( sex == 1 ){
      return geneticPositionsInMales[i];
    } else {
      return geneticPositionsInFemales[i];
    }
  }
 
  void PrintHaplotypeFrequencies(){
    
    int i,b,j;
    for(i=0; i< nLoci;i++){     
      b = locusInHaplotypeBlockNumber[i];
      for( j = 0; j< nHaplotypes[ b] ; j++){
	cout << "NSNP : " << nSNPInHaplotypes[b] << " H: "<< Haplotype(b,j) << " " <<  haplotypeFrequencies[b][j] ;
	cout << " "  << cumulHaplotypeFrequencies[b][j]<< endl;
      }
      cout << endl;      
      i += nSNPInHaplotypes[ b ] - 1;
    }
  }
  
  void PrintAlleleFrequencies(){
    
    int i,j;
    for(i=0; i< nLoci;i++){
      
      for( j = 0; j< NAlleles(i); j++){
	cout << LocusPositionInItsHaplotypeBlock(i) << " " << i+1 << "-" << j+1 << ":" << AlleleFrequency(i,j) <<endl;
      }
    }
  }


};
