/*Copyright 2006 Mathieu Lemire*/
#include "Prefile.hh"

Prefile::Prefile(char * file , bool prefile = 0, bool dnacol = 0){

  int i,ped, ifoo ;
  double ffoo;
  char ligne [L_LENGTH];
  string foo, curped="", lastped = "";
  int * temp;
  
  int nIndiv=0, nPedi=0;
 
  int curncol=0, curline=0;;

  istringstream tampon;

  ifstream FILE (file, ios::in);
  if( !FILE ){
    cout << file << " : no such file \n";
    exit(-1);
  } else {
    cout << "<prefile>: Opening file " <<  file << " for reading." << endl;
  } 

  nColumns=0;
  while( 1 ){
    if( FILE.eof() ) break;
    FILE.getline(  ligne, L_LENGTH);
    curline++;
    istringstream L (ligne);
   
    if( L >> curped ){   
      curncol=1;
      nIndiv++;
      
      if( curped != lastped && curped.size() > 0 ){
	nPedi++;
	lastped = curped;
      }
      while( L >> ffoo ) curncol++;
      if( nColumns == 0 ){
	nColumns = curncol;
	cout << "  The file " << file << " has " << nColumns << " columns " << endl;
	cout << "   (ignoring trailing character strings";
	L.clear();
	L.str(ligne);
	for( i = 0; i<nColumns;i++) L >> foo;
	while( L >> foo )
	  cout << " " << foo;
	cout << ")." << endl;
	
      } else if( nColumns != curncol ){
	cout << "Inconsistent number of columns (" << curncol << ") in line " << curline << endl;
	exit(-1);
      }
    }
  }


  cout << "  Read " << nIndiv << " individuals in " << nPedi << " pedigrees." << endl;

  cout << "  Assuming that the trait locus genotypes are found in columns " << nColumns - 3 - int(dnacol) << "," << nColumns -2 - int(dnacol) << ";" << endl;
  cout << "  assuming that the descent marker genotypes are found in columns " << nColumns - 1 -int(dnacol) << "," << nColumns - int(dnacol)  << "." << endl;

  if( dnacol ){
    cout << "  Column " << nColumns << " indicates wether or not the individual can be genotyped." << endl;
  } else {
    cout << "  All individuals in file " << file << " will have genotypes." << endl;
  }
  
  
  
  FILE.clear();
  FILE.seekg(0);

  nPedigrees = nPedi;
  nIndividuals = nIndiv;
  pedigreeName = new string [nIndiv];
  individualName= new string [nIndiv];
  fatherName= new string [nIndiv];
  motherName= new string [nIndiv];
  sex= new int [nIndiv];
  hasDna = new bool [nIndiv];
  isFounder = new bool [nIndiv];
  motherIs = new int [nIndiv];
  fatherIs = new int [nIndiv];

  restOfLine= new string [nIndiv];
    
  pedigreeBounds = new int [nPedi + 1];

  ped = -1;

  for(i=0; i< nIndiv;i++){
    if( FILE.eof() ) break;
    FILE.getline( ligne, L_LENGTH);
    istringstream L (ligne);
    L >> pedigreeName[i] >> individualName[i] >> fatherName[i] >> motherName[i];
    if( ! prefile ){
      L >> foo >> foo >> foo ;
    } 
    L >> sex[i];
    if( sex[i] != 1 && sex[i] != 2 ){
      cout << "Wrong sex for " << pedigreeName[i]<<"-"<<individualName[i]  << ": read the value " << sex[i]  << endl;
      cout << " (only the values 1 and 2 are allowed)" << endl;
      exit(-1);
    }
    if( pedigreeName[i] != lastped ){
      ped++;
      pedigreeBounds[ ped ]=i;
      lastped = pedigreeName[i];
    }      

    if( fatherName[i] != string("0") ){
      isFounder[i] = 0;
    } else {
      isFounder[i] = 1;
    }

    ostringstream O;
    
    prefile?ifoo=5:ifoo=8;

  
    if( L >> foo ){
      if( prefile )
	O << foo; // if postmakeped, this is proband status
      ifoo++;
    }
    while( ifoo < nColumns && L >> foo ){
      O << " " << foo;
      ifoo++;
    }
    restOfLine[i]=O.str();

    if( dnacol ){
      // the availability code is the last column
      tampon.clear();
      tampon.str(foo);
      if( tampon >> ifoo && ( ifoo == 0 || ifoo == 1 || ifoo == 2) ){
	hasDna[i] = bool( ifoo );
      } else {
	cout << "Problem reading the dna availability column: values should be 0,1 or 2 but"<<endl;
	cout << " read the value \"" << foo << "\" in line " << i+1 << " of file " << file << "." << endl;
	exit(-1);
      }
	       
      //cout << hasDna[i] << endl;
    } else {
      hasDna[i]=1;
    } 
  }
    
  pedigreeBounds[nPedi]=i;

  if( DEBUG ){
    for( ped = 0; ped<nPedi; ped++){
      for( i =  pedigreeBounds[ped] ; i<  pedigreeBounds[ped+1 ]; i++){
	if( !isFounder[i] )
	  cout << ped << " : " <<  pedigreeName[i] << "-" << individualName[i] << " "<< fatherName[i]<< endl;
      }
    }
  }
}


void Prefile::FindParents(){
    
  int i,j, k;
    
  for( i=0;i<nIndividuals;i++){
    fatherIs[i]=0;
    motherIs[i]=0;
  }

  for( i=0;i<nPedigrees;i++){
    for(j=pedigreeBounds[i]; j<pedigreeBounds[i+1]; j++){
      if( isFounder[j] ){
	motherIs[j]=-1;
	fatherIs[j]=-1;
      } else {
	for( k=pedigreeBounds[i]; k<pedigreeBounds[i+1]; k++){
	  if( k!=j ){
	    if( !fatherIs[j] && individualName[k] == fatherName[j] ) fatherIs[j]=k;
	    if( !motherIs[j] && individualName[k] == motherName[j] ) motherIs[j]=k;
	  }
	  if( fatherIs[j]*motherIs[j] ) break;
	}
      }
    }
  }
  if( DEBUG){
    for( i=0;i<nIndividuals;i++){
      cout << i << " " <<  pedigreeName[i] << "-" << individualName[i] << " ";
      cout << fatherName[i] << " (" << fatherIs[i] << ") --  ";
      cout << motherName[i] << " (" << motherIs[i] << ")" <<endl;
    }
  }
	

}
		



/*
main(){

  Prefile PF ("simpre.dat", 1280, 225);
  PF.FindParents();
}
*/

 

 
	
    

